#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

class Debate < ActiveRecord::Base
  include Sluggable
    
  belongs_to :creator, :class_name => "User", :foreign_key => "created_by"
  belongs_to :last_editor, :class_name => "User", :foreign_key => "updated_by"
    
  belongs_to :department, :class_name => "Organization", :foreign_key => "organization_id"
  
  has_many :stages, :class_name => "DebateStage", :foreign_key => "debate_id", :order => "position", :dependent => :destroy
  accepts_nested_attributes_for :stages, :allow_destroy => true
  validates_associated :stages

  has_many :debate_entities, :order => 'position', :dependent => :destroy
  has_many :entities, :through => :debate_entities, :source => :organization, :order => 'position'

  has_many :attachments, :as => :attachable, :class_name => '::Attachment', :dependent => :destroy, :foreign_key => 'attachable_id'
  
  has_many :comments, :as => :commentable, :order => "comments.created_at DESC", :dependent => :destroy
  
  has_many :votes, :as => :votable, :dependent => :destroy
  
  has_many :arguments, :as => :argumentable, :dependent => :destroy
  
  belongs_to :page, :class_name => "Page", :foreign_key => "page_id"
  
  belongs_to :news, :class_name => "News", :foreign_key => "news_id"
  
  
  named_scope :published, lambda { |*args| {:conditions => ["published_at IS NOT NULL AND debates.published_at <= ?", (args.first || Time.zone.now)]}}
  named_scope :translated, lambda {{:conditions => "coalesce(title_#{I18n.locale}, '') <> '' AND coalesce(body_#{I18n.locale}, '') <> '' "}}
  named_scope :featured, lambda {{:conditions => {:featured => true}, :order => "published_at DESC", :limit => 6}}      

  validates_presence_of :title_es, :hashtag
  validates_length_of :title_es, :title_eu, :title_en, :maximum => 400, :allow_blank => true  

  validates_presence_of :organization_id
  
  validates_length_of :multimedia_dir, :maximum => 240, :allow_blank => false # dejamos sitio para poner /debates antes de multimedia_dir
  validates_format_of :multimedia_dir, :with => /^[a-z0-9_]+$/i

  validates_length_of :hashtag, :maximum => 240, :allow_blank => false
  validates_uniqueness_of :hashtag

  before_save :set_default_image #important: always before mount_uploader, otherwise doesn't work

  mount_uploader :cover_image, DebateImageUploader
  mount_uploader :header_image, DebateHeaderUploader
  
  translatable_columns :title, :body, :description

  acts_as_taggable

  before_validation :check_hashtag
  
  before_create :create_debate_tag  
  after_update  :update_debate_tag

  # Preserve this order
  include DraftUtils::InstanceMethods
  before_save :sync_draft_and_published_at # definido en draft_utils.rb
  # / Preserve this order
  
  include Tools::Content

  # Los debates se pueden votar
  include Tools::Votable
  
  # Usamos los tags para asignar área al debate
  include Tools::WithAreaTag
  include Tools::WithPoliticiansTags

  # Los debates, igual que las noticias tienen directorio multimedia
  include Tools::Multimedia

  include Elasticsearch::Base

  before_save :set_and_create_multimedia_path
  
  before_save :set_published_at
  after_save :destroy_non_active_stages

  def init_stages
    if self.stages.empty?
      DebateStage::STAGES.each_with_index do |stage, i|
        self.stages.build(:label => stage.to_s, :starts_on => Date.today + i.months, :ends_on => Date.today + (i+1).months - 1.day)
      end
    end
  end

  # Indica si un documento está publicado
  def published?
    !published_at.nil? && published_at <= Time.now
  end

  # Indica si el debate es público (no está en borrador)
  def is_public?
    !self.published_at.nil?
  end
  
  # Indica si el debate está traducido a <tt>lang_code</tt>
  # Los idiomas disponibles son <tt>Document::LANGUAGES</tt>
  def translated_to?(lang_code)
    self.send("title_#{lang_code}").present? && self.send("body_#{lang_code}").present?
  end
  
  # Indica si hay foto de portada. Sólo se usa en caso de que no haya video de portada
  # Para compatibilidad con Document (se usa en Tools::Multimedia)
  def has_cover_photo?
    self.cover_image.present?
  end
  
  
  #
  # Fases del debate
  #
  
  # Devuelve true si la fecha de inicio de la primera fase es futura
  def future?
    self.stages.first.starts_on > Date.today
  end
  
  # Devuelve true si ya ha acabado la última fase
  def finished?
    self.stages.last.ends_on < Date.today
  end
  
  # Devuelve el estado actual como objeto DebateStage
  def current_stage
    current = if self.published?
      if self.future?
        # Todavía no ha llegado la fecha de la presentación
        self.stages.first
      elsif  self.finished?
        self.stages.last
      else
        this=self.stages.detect {|s| s.starts_on <= Date.today && s.ends_on >= Date.today}
        if this.nil?
          self.stages.detect {|s| s.starts_on <= Date.today || s.ends_on >= Date.today}
        end
        this
      end
    else
      self.stages.first
    end
    current
  end

  # Definimos métodos para acceder a cada una de las fases.  
  DebateStage::STAGES.each do |label|
    define_method("#{label}_stage") do
      self.stages.detect {|s| s.label.eql?(label.to_s)}
    end
  end
  
  # Fin de los métodos relacionados con las fases.

  # Para descatar una noticia en un debate hay que ponerle el tag "_destacado_<hashtag>"
  def featured_tag_name_es
    "_destacado#{self.hashtag}"
  end
  
  def featured_news
    News.published.tagged_with(self.featured_tag_name_es).find(:all, :order => "published_at DESC", :limit => 8)
  end
    
  def related_news
    News.published.tagged_with(self.hashtag).find(:all, :order => "published_at DESC", :limit => 8)
  end

  def approved_headlines     
    Headline.published.translated.tagged_with(self.hashtag).find(:all, :limit => 10)
  end       
  
  # Para compatibilidad con las propuestas ciudadanas
  # (se usan cuando se muestra la información sobre un vote)
  # 2DO
  def has_comments
    true
  end
  alias_method :has_comments?, :has_comments
  
  def comments_closed?
    !self.stages.find_by_label('discussion').is_current?
  end

  def author
    self.organization
  end
  
  def author_name
    self.author.name
  end
  
  def organization
    department
  end

  # El tag del debate coincide con su hashtag (# incluido)
  def tag_name
    self.hashtag
  end

  def total_participation
    self.arguments.published.size + self.votes_count
  end

  # Para compatibilidad con los documentos.
  # Nos permite compartir partials ente Document y Debate 
  # (por ejemplo el de attachments y el de traducciones)
  def class_name
    self.class.to_s
  end  

  def tags_without_hashtag
    if hashtag_tag = Tag.find_by_name_es(self.hashtag)
      tags.public - [hashtag_tag]
    else
      tags.public
    end
  end

  private

  # Add # at the begining of the hashtag unless present
  def check_hashtag
    if self.hashtag.present? && self.hashtag.match(/^\#/).nil?
      self.hashtag = '#'+self.hashtag
    end
    self.hashtag
  end 
  
  # El tag que corresponde al debate se genera a partir del hashtag.
  def create_debate_tag
    self.tag_list_es.push self.tag_name
  end
  
  #
  # Cambiar el nombre del tag del debate si ha cambiado el hashtag.
  #
  # NOTA: Hay que hacer el cambio en un after update cuando los tags de tag_list_es ya están asignados.
  # Si se hace este cambio en un before_save, se borra el tag con el nombre viejo y se crea el nuevo.
  # Pero esto no es correcto porque se perderá la relación con los demás contenidos.
  #
  def update_debate_tag
    if self.hashtag_changed?
      old_tag_name = self.changes["hashtag"].first
      if old_tag = Tag.find_by_name_es(old_tag_name)
        new_tag_name = self.tag_name
        old_tag.update_attributes(:name_es => new_tag_name, :name_eu => new_tag_name, :name_en => new_tag_name)
      end
    end
  end

  # 2DEL
  # # Asignamos el valor de ends_on para las fases
  # # Desde el formulario sólo se indica el valor de starts_on para cada fase.
  # # La feca fin de inicio de una fase es la fecha fin de la fase anterior.
  # # El valor de ends_on se indica sólo para la última fase.
  # # NOTA: Hay que hacer esta asignación antes de las validaciones.
  # def set_stages_ends_on
  #   if self.stages.present?
  #     self.presentation_stage.ends_on = self.discussion_stage.starts_on
  #     self.discussion_stage.ends_on = self.contribution_stage.starts_on    
  #     self.contribution_stage.ends_on = self.conclusions_stage.starts_on
  #   end
  # end
  # # Comprobar que las fechas de inicio de las fases están en el orden correcto.
  # # Si alguna no es correcta, cambia el valor de correct para indicar que 
  # # hay valor que no es válido.
  # def check_stages_dates
  #   if self.stages.present?
  #     
  #     correct = true
  #     self.stages.each do |stage|
  #       correct = stage.send("has_correct_starts_on?", self.stages) if correct
  #     end
  #   
  #     unless correct 
  #       self.errors.add(:stages, "no es válido")
  #     end
  #     
  #     return correct
  #   end
  #   
  #   true
  # end  

  # Si el debate no está en borrador, se asigna como fecha de publicación, 
  # la fecha de inicio de la primera fase.  
  def set_published_at
    unless self.draft == "1"
      self.published_at = self.presentation_stage.starts_on
    end  
    true
  end
  
  def destroy_non_active_stages
    self.stages.find_all {|s| !s.active}.each {|s| s.destroy}
  end

  def set_default_image
    last_indices = (Debate.find(:all, :order => "updated_at DESC") - [self]).map{|a| a.cover_image.url.match(/.*([0-9]).jpg/)[1] if a.cover_image.present? && a.cover_image.url.match(/default/)}.compact
    # puts "LAST INDEXES #{last_indexs.inspect}"
    index = last_indices.empty? ? 0 : last_indices.first.to_i + 1
    index = (index > 3) ? 0 : index
    # puts "INDEXXXXXXX #{index}"
    unless self.cover_image.present?
      self.cover_image = File.open(File.join(RAILS_ROOT, 'public/images/default/', "debate_default_0#{index}.jpg"))
    end
    unless self.header_image.present?
      self.header_image = File.open(File.join(RAILS_ROOT, 'public/images/default/', "debate_header_default_0#{index}.jpg"))
    end
    return true
  end
  
end
