#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase que engloba todos los tipos de usuario. Los diferentes tipos son subclase de esta clase.
# Aquí se definen todos los métodos comunes para todos los tipos de usuario, y luego se 
# sobreescriben en la subclase correspondiente, para aquellos tipos en los que el comportamiento
# por defecto no es válido
require 'digest/sha1'
class User < ActiveRecord::Base
  has_many :arguments, :dependent => :destroy
  has_many :comments, :order => "created_at DESC", :dependent => :nullify
  has_many :proposals
  has_many :votes, :dependent => :destroy

  # HQ
  # has_many :questions, :dependent => :destroy
  # has_many :answer_requests, :dependent => :destroy
  
  # has_many :subscriptions, :dependent => :destroy
  
  has_many :schedules_permissions, :class_name => 'SchedulesPermission', :dependent => :destroy
  has_many :schedules, :through => :schedules_permissions
  has_many :editable_schedules, :through => :schedules_permissions, :source => :schedule, :conditions => "can_edit = 't'"
  has_many :changable_schedules, :through => :schedules_permissions, :source => :schedule, :conditions => "can_change_schedule = 't'"
  
  has_many :permissions, :dependent => :destroy  
  has_many :session_logs, :dependent => :destroy
  has_many :event_alerts # no pongo :dependent => :destroy porque solo queremos borrar las que aun no se han enviado

  # oauth-plugin
  has_many :client_applications
  has_many :tokens, :class_name=>"OauthToken",:order=>"authorized_at desc",:include=>[:client_application]
  # /oauth-plugin  
  
  has_many :followings
  has_many :following_areas, :through => :followings, :source => :followed, :source_type => 'Area'
  has_many :following_politicians, :through => :followings, :source => :followed, :source_type => 'Politician'
  
  has_many :notifications, :dependent => :destroy
  
  mount_uploader :photo, PhotoUploader

  # Estados de los usuarios:
  # pendiente: el usuario se ha dado de alta y no ha confirmado su registro (para ciudadanos que se registran desde la web)
  # aprobado: el usuario se ha dado de alta y ha confirmado su registro (para los ciudadadanos) 
  #           o los admin han cambiado su estado (para los demás usuarios)
  # vetado: los admin han vetado el acceso del usuario. 
  #         Si es político, en la web no sale ni su página de político ni él sale en el equipo de área al que está asignado.
  #         Este usuario no puede logearse en la web
  #  eliminado: un ciudadanno que se ha dado de baja (ha eliminado su perfil). 
  #             Se ven los contenidos que él ha creado pero no salen sus datos personales
  # ex-cargo: político que ya no está en el cargo que ocupaba. 
  #           Sale su página y su actividad pero no se le pueden hacer preguntas ni se le puede seguir
  User::STATUS = %w(pendiente aprobado vetado eliminado ex-cargo)
  
  User::TYPES = {'Person' => 'Registrado', 'Journalist' => 'Periodista', 'Politician' => 'Político', 
    'Admin' => 'Administrador', 'Colaborator' => 'Colaborador', 
    'DepartmentEditor' => 'Jefe de prensa', 'StaffChief' => 'Jefe de gabinete', 'DepartmentMember' => 'Miembro de departamento',
    'StreamingOperator' => 'Operador de streaming', 'RoomManager' => "Responsable de sala"}

  # Usuarios que se consideran personal de Lehendakaritza
  User::STAFF = %w(Admin DepartmentEditor StaffChief)
  
  # Virtual attribute for the unencrypted password
  attr_accessor :password

  validates_presence_of     :email,                      :unless => Proc.new {|user| user.is_twitter_user? || user.is_facebook_user?} 
  validates_presence_of     :name
  validates_presence_of     :type
  validates_presence_of     :password,                   :if => :password_required?
  validates_presence_of     :password_confirmation,      :if => :password_required?
  validates_length_of       :password, :within => 4..40, :if => :password_required?
  validates_confirmation_of :password,                   :if => :password_required?
  # validates_length_of       :email,    :within => 3..100,:unless => :email_blank
  validates_uniqueness_of   :email,                      :case_sensitive => false, :unless => :email_blank
  validates_format_of       :email,                      :with => /^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i, :unless => :email_blank
  validates_format_of :url, :with => /(^$)|(^(http|https):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(([0-9]{1,5})?\/.*)?$)/ix , :message => I18n.t('comments.blog_url_incorrect'), :allow_blank => true
  # validates_format_of       :url,                        :with => URI::regexp(%w(http https)), :allow_blank => true
  
  
  # prevents a user from submitting a crafted form that bypasses activation
  # anything else you want your user to change should be added here.
  # attr_accessible :email, :password, :password_confirmation, :name, :last_names, :url, :media
  attr_protected :type
  
  named_scope :approved, :conditions => "status='aprobado'"
  named_scope :pending, :conditions => "status='pendiente'"
  # This should be in politician.rb but then we couldn't use area.users.with_agenda
  named_scope :with_agenda, :conditions => ["politician_has_agenda=?", true]
  
  before_save :reset_unnecessary_fields
  before_save :encrypt_password
  before_create :set_default_status  
  before_validation :nullify_url_if_necessary
  before_save :reset_permissions_if_role_changed
  before_save :reset_schedules_permissions_if_department_changed
  before_destroy :destroy_pending_alerts
  
  after_save :create_tag_if_changed_to_politician
  
  after_save :reset_official_commenters
  after_destroy :reset_official_commenters
  
  def self.irekia_robot
    find_by_email("sn@irekia.euskadi.net")
  end
  
  # Devuelve un array con los tipos de usuarios que pertenecen a algún departamento
  def self.with_department
    User::TYPES.select {|k, v| k.constantize.new.has_department?}.collect {|u| u[0].to_s}
  end
  
  # Autentifica al usuario por su email, status y contraseña. Devuelve el usuario o nil.
  def self.authenticate(email, password)
    u = find_by_email_and_status(email, 'aprobado') # need to get the salt
    u && u.authenticated?(password) ? u : nil
  end
  
  # Autentifica al usuario por su id y contraseña encriptada. Devuelve el usuario o nil
  def self.authenticate_from_url(user_id, crypted_password)
    u = find_by_id_and_status_and_crypted_password(user_id, "aprobado", crypted_password)
  end
  
  cattr_accessor :official_commenters
  def self.official_commenters
    # This is expensive, so we memoize it. Memo will be flushed if there is a user type change
    # or a permission is changed.
    # Note: class variable localization only works in production by default
    # http://stackoverflow.com/questions/9720520/do-ruby-class-variables-get-cleared-between-rails-requests
    @@official_commenters ||= User.approved.find(:all).select {|u| u.is_official_commenter?}
    # @@official_commenters = [User.find(4067)]
  end
  
  
  # Nombre tal y como aparecerá en las páginas públicas
  def public_name
    if name.eql?('deleted')
      I18n.t('users.eliminado')
    else  
      "#{name} #{last_names}"
    end  
  end
  
  # Nombre interno, principalmente para el "staff" que tiene nombres públicos
  # con los que no se les puede identificar fácilmente
  def internal_name
    internal_name = public_name
    internal_name << " (#{telephone})" unless telephone.blank?
    return internal_name
  end
  
  # Encripta los datos con el salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encripta la contraseña con el salt del usuario
  def encrypt(password)
    self.class.encrypt(password, salt)
  end

  # Determina si el usuario está autentificado
  def authenticated?(password)
    crypted_password == encrypt(password)
  end

  def remember_token?
    remember_token_expires_at && Time.now.utc < remember_token_expires_at 
  end

  # These create and unset the fields required for remembering users between browser closes
  def remember_me
    remember_me_for(2.weeks)
  end

  def remember_me_for(time)
    remember_me_until time.from_now.utc
  end

  def remember_me_until(time)
    self.remember_token_expires_at = time
    self.remember_token            = encrypt("#{email}--#{remember_token_expires_at}")
    save(false)
  end

  def forget_me
    self.remember_token_expires_at = nil
    self.remember_token            = nil
    save(false)
  end
  
  def is_admin?
    self.is_a?(Admin)
  end
  
  def is_citizen?
    self.is_a?(Person) || self.is_a?(Journalist)
  end

  def published?
    self.status.eql?('aprobado')
  end
  
  # Indica si tiene permiso para acceder a la administración de los recursos de tipo <tt>doc_type</tt>. Por defecto, 
  # sólo los usuarios de tipo Admin pueden acceder a todos los módulos. En las subclases se concretan los permisos 
  # para cada tipo de usuario
  # ==== Ejemplos:
  # - current_user.can_access?("news")
  # - current_user.can_access?("photos")
  def can_access?(doc_type)
    if doc_type.eql?("users")
      can?("administer", "permissions")
    else
      [Admin].include?(self.class) 
    end
  end

  # Indica si puede modificar recursos de tipo <tt>doc_type</tt>. Por defecto, sólo los usuarios de tipo Admin 
  # pueden modificar todos los módulos. En las subclases se concretan los permisos para cada tipo de usuario.
  # ==== Ejemplos:
  # - current_user.can_edit?("news")
  # - current_user.can_edit?("photos")  
  def can_edit?(doc_type)
    [Admin].include?(self.class) 
  end
  
  # Indica si puede crear recursos de tipo <tt>doc_type</tt>. Por defecto, sólo los usuarios de tipo Admin 
  # pueden modificar todos los módulos. En las subclases se concretan los permisos para cada tipo de usuario.
  # ==== Ejemplos:
  # - current_user.can_create?("news")
  # - current_user.can_create?("photos")
  def can_create?(doc_type)
    [Admin].include?(self.class) 
  end
  
  # Indica si el usuario es miembro del "staff"
  def is_staff?
    User::STAFF.include?(self.class.to_s)
  end
  
  # Indica si puede acceder a alguna de las secciones de la administracion
  def has_admin_access?
    self.is_staff? || ["Colaborator", "StreamingOperator", "DepartmentMember", "RoomManager"].include?(self.class.to_s)
  end
    
  # Indica si los comentarios de este usuario se consideran oficiales y, por lo tanto,
  # se aprueban automáticamente y se muestran en color destacado
  def is_official_commenter?
    self.is_staff? || self.can?('official', 'comments')
  end

  # Indica si se debe especificar departamento para este usuario
  def has_department?
    self.respond_to?("department")
  end
  
  # Devuelve un array de los organismos a los que está suscrito. Sólo relevante para periodistas
  def organization_ids
    []
  end

  # HQ
  # def has_requested_answer(question)
  #   answer_requests.find(:first, :conditions => ["question_data_id=?", question.question_data.id]).present?
  # end
  # 
  # def approved_and_published_questions
  #   self.questions.approved.published
  # end
  # 
  # Para reaprovechar los métodos comunes en la actividad de un usuario
  # %w(answers).each do |item|
  #   define_method(item) do
  #     nil
  #   end
  # end
  

  # Returns the list with schedules for which a separate create event button is shown
  def editable_schedules_with_create_option
    self.editable_schedules.find(:all, :conditions => 'department_id IS NULL')
  end

  def editable_schedules_with_radio
    self.editable_schedules - self.editable_schedules_with_create_option
  end


  # Permisos para las agendas especiales
  def has_schedule_permissions?
    !self.schedules_permissions.blank?
  end
  
  # Indica si puede ver la agenda especial <tt>schedule_id</tt>
  def can_see_schedule?(schedule_id)
    self.schedule_ids.include?(schedule_id)
  end
  
  # Indica si puede ver los eventos a los que asiste el lehendakari
  def can_see_l_schedule?
    if s = Schedule.lehendakari
      return self.can_see_schedule?(s.id)
    end
    false
  end
  
  # Indica si el usuario puede modificar los eventos de la agenda <tt>schedule_id</tt>
  def can_edit_schedule?(schedule_id)
    res = false
    if perm = self.schedules_permissions.find_by_schedule_id(schedule_id)
      res = perm.can_edit?
    end
    res
  end

  # Indica si el usuario puede traspasar los eventos de la agenda <tt>schedule_id</tt>  a la agenda compartida.
  def can_change_schedule?(schedule_id)
    res = false
    if perm = self.schedules_permissions.find_by_schedule_id(schedule_id)
      res = perm.can_change_schedule?
    end
    res
  end
  
  # Indica si el colaborador tiene permiso de tipo <tt>perm_type</tt> en los contenidos de tipo <tt>doc_type</tt>.
  # Los tipos de contenidos y los correspondientes permisos se pueden consultar en #Permission
  # ==== Ejemplos:
  # - current_user.can?("create", "news")
  # - current_user.can?("administer", "permissions")
  def can?(perm_type, doc_type)
    # Admin tiene permiso para todo menos para gestionar usuarios y repartir permisos
    if self.is_a?(Admin) && !%w(users permissions).include?(doc_type)
      true
    elsif (![DepartmentMember, Politician].include?(self.class)) && doc_type.eql?("events") &&  %w(create_irekia create_private).include?(perm_type)
      # Los permisos para publicar noticias en irekia, agencia y privados, solo se ponen para los miembros
      # de departamento y políticos. Para los demas, el permiso es el mismo de crear evento
      self.can_create?(doc_type)
    else
      permission?(perm_type, doc_type)
    end
  end
  
  # Indica si el usuario tiene permiso de tipo <tt>perm_type</tt> en el modulo <tt>doc_type</tt>.
  # Para ver los valores posibles de estas dos variables, ver el comentario inicial de la clase Permission
  def permission?(perm_type, doc_type)
    conditions = ["module=?", doc_type]
    case
    when perm_type.eql?("edit")
      # El permiso de crear lleva consigo el de modificar
      conditions[0] << " AND action IN ('edit', 'create')"
    when !perm_type.eql?("access")
      # access significa cualquier tipo de permiso sobre el modulo
      conditions[0] << " AND action=?"
      conditions << perm_type
    end
    permissions.exists?(conditions)
  end
  
  # Devuelve un array con todos los permisos que tiene un usuario
  # Incluye:
  # * Los permisos heredados del tipo de usuario
  # * Los permisos dados manualmene a través de la clase #Permission
  # * Los permisos dados manualmente a través de la clase #SchedulesPermission. Solo se incluyen los 
  #   permisos sobre la agenda del departamento al que pertenece el usuario (si es que pertenece a alguno)
  #   porque son los únicos que se pueden modificar a través de ese formulario
  def all_editable_permissions
    fp = self.permissions.dup

    if self.has_department_schedule?
      schedules_permissions.find(:all).each do |sp|
        if sp.can_edit?
          fp << Permission.new(:user_id => self.id, :module => "schedule_events", :action => "can_edit", :not_editable => false)
        end
        if sp.can_change_schedule?
          fp << Permission.new(:user_id => self.id, :module => "schedule_events", :action => "can_change_schedule", :not_editable => false)
        end
      end
    end

    if self.class.respond_to?("inherited_permissions")
      self.class.inherited_permissions.each do |perm|
        fp << Permission.new(perm.merge(:not_editable => true, :user_id => self.id))
      end
    end
    return fp
  end
  
  # Devuelve un hash con todos los permisos de este usuario, tanto los heredados por el role como los
  # obtenidos a traves de Permission.
  # Formato:
  #   <tt>{"comments"=>["edit", "official"], "events"=>["create_private", "create_irekia"], "news"=>["create"]}</tt>
  #
  def all_permission_by_module
    all_perms = {}
    all_editable_permissions.each {|p| all_perms.has_key?(p.module) ? all_perms[p.module] << p.action : all_perms[p.module] = [p.action]}
    return all_perms
  end
  
  # Devuelve los mismos permisos que <tt>all_editable_permissions</tt> pero en el siguiente formato, 
  # adecuado para los campos del formulario
  # <tt>[perm[news][create], perm[events][create_private]]</tt>
  def permissions_for_form_array
    output = self.all_editable_permissions.collect {|p| "perm[#{p.module}][#{p.action}]"}
    # logger.info "permissions_for_form_array #{output.inspect}"
    return output
  end
  
  # Actualiza los permisos del usuario, tanto los de los eventos especiales como los permisos generales
  def update_permissions(new_permissions)
    new_permissions ||= {}
    self.transaction do
      logger.info "Nuevos permisos..... #{new_permissions.inspect}"
      if new_schedules_permissions = new_permissions.delete(:schedule_events)
        logger.info "Schedules permissions......... #{new_schedules_permissions.inspect}"
        # Con esta interfaz simplificada, solo podemos asignar permisos en agendas especiales
        # en el departamento al que pertenece el usuario. El resto hay que hacerlo a mano en SQL
        if has_department_schedule?
          # Mantenemos los permisos que le hemos dado fuera de su departamento
          self.schedules_permissions = self.schedules_permissions.find(:all, :conditions => "schedule_id <>#{self.department.schedule.id}")
          new_schedules_permissions_with_sym_keys = Hash[*new_schedules_permissions.collect {|k, v| [k.to_sym, v]}.flatten]
          self.schedules_permissions.build({:schedule_id => self.department.schedule.id, :can_edit => false, :can_change_schedule => false}.merge(new_schedules_permissions_with_sym_keys))
        end
      else
        if self.has_department_schedule?
          self.schedules_permissions.delete(self.schedules_permissions.find(:all, :conditions => "schedule_id =#{self.department.schedule.id}"))
        end
      end
    
      self.permissions.clear
      logger.info "Resto de permisos......... #{new_permissions.inspect}"
      new_permissions.each do |mod, permissions|
        permissions.each do |perm|
          self.permissions.build(:module => mod, :action => perm[0]) if perm[1].to_i == 1
        end
      end
      self.save
    end
  end
  
  # Indica si el usuario existe una agenda privada para el departamento al que pertenece el usuario,
  # si es que pertenece a alguno
  def has_department_schedule?
    self.has_department? && self.department_id.present? && Schedule.exists?(:department_id=> self.department_id)
  end

  attr_writer :stream_flow_ids
  # Los RoomManagers tienen <tt>stream_flow_ids</tt> gracias a <tt>has_many :stream_flows, :through => :room_managements</tt>, 
  # que son las salas de las que son responsables. Necesitamos definir para el resto de usuarios este método para vaciar 
  # el mapping cuando cambiamos un usuario de RoomManager a otro role.
  def stream_flow_ids=(val)
    RoomManagement.delete_all("room_manager_id = #{self.id}") unless (self.new_record? || self.is_a?(RoomManager))
  end

  def is_outside_user?
    is_twitter_user? || is_facebook_user?
  end
  
  def is_twitter_user?
    screen_name.present?
  end

  def is_facebook_user?
    fb_id.present?
  end

  # irekia-auth API related methods
  def role4api
    self.type.to_s
  end
  
  def name4api
    self.name
  end
  
  def lastname4api
    self.last_names
  end
  
  def postal_code4api
    self.zip
  end

  def province4api
    self.state
  end

  def city4api
    self.city
  end

  def follows?(item)
    self.send("following_#{item.class.name.downcase.pluralize}").include?(item) if item.present?
  end
  
  def not_follows?(item)
    !self.follows?(item)
  end
  
  def active_notifications_for(item)
    self.notifications.pending.find(:all, :conditions => "notifiable_id=#{item.id} AND notifiable_type='#{item.class.base_class}'")
  end
  
  def deactivate_account
    self.status = 'eliminado'
    self.email = "deleted_#{self.id}@email.com"
    self.remove_photo!
    self.name = 'deleted'
    self.last_names = 'deleted'
    self.screen_name = nil
    self.fb_id = nil
    self.asecret = nil
    self.atoken = nil
    pass = User.random_password
    self.password = pass
    self.password_confirmation = pass
    if self.is_a?(Journalist)  
      self.media = 'deleted'
      self.last_names = 'deleted'
    end  
    self.comments.map{|c| c.update_attribute(:name, 'deleted')}
    self.save
  end  
  
  # Genera un password aleatorio
  # http://snippets.dzone.com/posts/show/2137
  def self.random_password(size = 5)
    chars = (('a'..'z').to_a + ('A'..'Z').to_a + ('0'..'9').to_a) - %w(i o 0 1 l 0 I O)
    (1..size).collect{|a| chars[rand(chars.size)] }.join
  end

  def approved?
    self.status.eql?('aprobado')
  end
    
  def approved_and_published_proposals
    self.proposals.approved.published
  end
  
  def approved_comments
    self.comments.approved
  end

  def approved_arguments
    self.arguments.published
  end   
  
  def approved_headlines
    []
  end
  
  # Esto lo necesito para poder aplicar el scope "leading" sobre las noticias del home de un usuario
  # porque en el home de usuario se usa la misma función que en el home de un área o de un político
  def tag_name_es
    "los usuarios no tienen tag"
  end
  
  # Acciones del político
  include Tools::WithActions
  
  

  protected
    # Encripta la contraseña
    def encrypt_password
      if self.is_a?(Journalist) && self.new_record?
        generate_random_password_if_empty
      end
      
      return if password.blank?
      self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{email}--") if new_record?
      self.crypted_password = encrypt(password)
      self.plain_password = password
    end
    
    # Por defecto los usuarios están aprobados. Se llama desde before_create.
    def set_default_status
      self.status = 'aprobado' unless self.status
    end
    
    # Resetea los permisos de la tabla permissions si el usuario cambia de role
    def reset_permissions_if_role_changed
      self.permissions.clear if self.type_changed?
    end
    
    # Resetea los permisos de las agendas de departamento si el usuario cambia de departamento
    def reset_schedules_permissions_if_department_changed
      self.schedules_permissions = self.schedules_permissions.select {|sp| sp.schedule.department_id.nil?} if self.department_id_changed?
    end
    
    def destroy_pending_alerts
      EventAlert.delete_all("spammable_id=#{self.id} AND sent_at IS NULL")
    end

    # Determina si el password es obligatorio
    def password_required?
      !type.eql?('Journalist') && !is_twitter_user? && !is_facebook_user? && (crypted_password.blank? || !password.blank?)
    end
    
    # Indica si el email está vacío
    def email_blank
      self.email.blank?
    end
    
    # Genera un password aleatorio
    def generate_random_password_if_empty
      self.password = Journalist.random_password if self.password.blank?
    end
    
    # Si el usuario deja la URL con el valor por defecto, se quita porque no es 
    # una URL válida
    def nullify_url_if_necessary
      self.url = nil if self.url.eql?("http://")
      return true
    end     

    # Crea el tag del político. Se llama desde after_create de un político o al cambiar el tipo de usuario.
    def create_new_politician_tag
      tag = Tag.create(:name_es => self.public_name, :name_eu => self.public_name, :name_en => self.public_name, :kind => 'Político', :kind_info => self.id.to_s, :translated => true)
      tag
    end

    # Si ha cambiado el tipo del usuario y el nuevo tipo es Politician, creamos el tag
    def create_tag_if_changed_to_politician
      if !self.respond_to?('tag') && self.type_changed? && self.type.eql?('Politician')
        tag = create_new_politician_tag()
        tag.taggings.create(:taggable_type => 'User', :taggable_id => self.id)
      end
    end
    
    def reset_official_commenters
      if self.type_changed? || User::STAFF.include?(self.type_was)
        User.official_commenters = nil
      end
    end
end
