require File.join(RAILS_ROOT, 'config', 'environment')

gem 'twitter'
require 'twitter'

namespace 'ogov' do
  
  # Recorre y envia la cola de alertas por enviar a periodistas 
  # Debería llamarse desde un cron job de la siguiente manera:
  # 7 * * * * sh /usr/app/ogov/batch_processes/process_unsent_alerts_for_journalists.sh
  desc "Process unsent alerts for journalists"
  task :process_unsent_alerts_for_journalists do
    alert_logger = Logger.new(File.dirname(__FILE__) + "/../../log/alerts.log") 
    alert_logger.info("#{Time.zone.now.to_s(:short)}: Processing emails queue for journalists...")
    
    pending_alerts_for_future_events = EventAlert.for_journalists.unsent.find :all, 
      :joins => :event,
      :conditions => ["send_at <= ? AND ends_at >= ?", Time.zone.now, Time.zone.now], 
      :order => "created_at"
    
    pending_alerts_for_future_events.each do |alert|
      send_alert(alert, alert_logger)
    end
    alert_logger.info("#{Time.zone.now.to_s(:short)}: Finished processing emails queue for journalists. ")
    alert_logger.info("===========")
  end
  
  # Recorre y envia la cola de alertas por enviar a responsables de salas y operadores de streaming
  # Debería llamarse desde un cron job de la siguiente manera:
  # 7 * * * * sh /usr/app/ogov/batch_processes/process_unsent_alerts_for_staff.sh
  desc "Process unsent alerts for staff"
  task :process_unsent_alerts_for_staff do
    alert_logger = Logger.new(File.dirname(__FILE__) + "/../../log/alerts.log") 
    alert_logger.info("#{Time.zone.now.to_s(:short)}: Processing emails queue for staff....")
    
    pending_alerts_for_future_events = EventAlert.for_staff.unsent.find :all, 
      :joins => :event,
      :conditions => ["send_at <= ? AND ends_at >= ?", Time.zone.now, Time.zone.now], 
      :order => "created_at"
    
    pending_alerts_for_future_events.each do |alert|
      send_alert(alert, alert_logger)
    end
    
    alert_logger.info("#{Time.zone.now.to_s(:short)}: Finished processing emails queue for staff. ")
    alert_logger.info("===========")
  end
  
  
  def send_alert(alert, alert_logger)
    begin
      alert_logger.info("\tSending alert #{alert.id} from event #{alert.event_id} to #{alert.spammable_type} #{alert.spammable.email} that was programmed for #{alert.send_at.to_s(:short)}")
      if alert.spammable_type.eql?('Journalist')          
        email = Notifier.create_journalist_event_alert(alert)
      else
        email = Notifier.create_staff_event_alert(alert)
      end
    rescue => err
      alert_logger.info("\talert #{alert.id}: There were some errors building email: #{err}")
    else
      begin
        Notifier.deliver(email)
      rescue => err_type
        alert_logger.info("\tThere were some errors sending event alert: " + err_type)
      else
        alert.update_attributes(:sent_at => Time.zone.now)
        # If the event was deleted and there is noone else to tell about it, remove it from the database
        if alert.event.deleted? && alert.event.alerts.unsent.count == 0
          alert_logger.info("\tDeleting event #{alert.event.id}")
          alert.event.destroy
        end
      end
    end
    alert_logger.info("-----------")
  end
  
  
  # Recorre y envia la cola de tweets sobre nuevas noticias y eventos.
  # Debería llamarse desde un cron job de la siguiente manera:
  # 22 * * * * sh /usr/app/ogov/batch_processes/tweet_pending_issues.sh
  desc "Tweet pending news and events"
  task :tweet_pending_issues do
    include ActionController::UrlWriter
    
    alert_logger = Logger.new(File.dirname(__FILE__) + "/../../log/alerts.log") 
    # alert_logger = Logger.new(STDOUT)
    alert_logger.info("#{Time.zone.now.to_s(:short)}: Processing tweets queue...")
    
    twitter_accounts = YAML::load_file(File.join(RAILS_ROOT, 'config', 'twitter.yml'))
    twitter_accounts.each do |account, account_info|
      
      # Twitter gem version 0.7.9
      oauth = Twitter::OAuth.new(account_info["token"], account_info["secret"])
      oauth.authorize_from_access(account_info["atoken"], account_info["asecret"])
      client = Twitter::Base.new(oauth)      
      # Twitter gem version 1.1.2
      # Twitter.configure do |config|
      #   config.consumer_key = account_info['token']
      #   config.consumer_secret = account_info['secret']
      #   config.oauth_token = account_info["atoken"]
      #   config.oauth_token_secret = account_info["asecret"]
      # end

      DocumentTweet.pending(account).each do |tweet|
        if tweet.document.past?
          alert_logger.info("\t Don't send tweet #{tweet.id} because document #{tweet.document_id} is quite old. Tweet is going to be deleted.")
          tweet.destroy
        else
          alert_logger.info("\tSending tweet #{tweet.id} of #{tweet.document.class} #{tweet.document.id} 
                               that was programmed for #{tweet.tweet_at.to_s(:short)}")
          begin
            if tweet.document.is_a?(News)
              url = url_for(:controller => "news", :action => "show", :id => tweet.document, 
                :locale => tweet.tweet_locale, :only_path => false, :host => Notifier::SERVICEURL)
              text = shorten("#{tweet.document.send("title_" + tweet.tweet_locale)}. ", 140 - 25) #25 because twitter shortens urls up to 25 chars
              text_and_url = "#{text} #{url}"
            else
              url = url_for(:controller => "events", :action => "show", :id => tweet.document, 
                :locale => tweet.tweet_locale, :only_path => false, :host => Notifier::SERVICEURL)
              event_date = I18n.localize(tweet.document.starts_at, :format => :short, :locale => tweet.tweet_locale)
              text = shorten("#{event_date}: #{tweet.document.send("title_" + tweet.tweet_locale)}. ", 140 - 25)
              text_and_url = "#{text} #{url}"
            end                      
            client.update(text_and_url)
          rescue => err_type
            # if err_type.to_s.eql?("(403): Forbidden - Status is over 140 characters.")
            #   alert_logger.info("\tWarning al mandar tweet #{tweet.id}: " + err_type)
            #   alert_logger.info("\tAun asi lo marcamos como mandado")
            #   tweet.update_attributes(:tweeted_at => Time.zone.now)
            # else
              alert_logger.info("\tError sending tweet #{tweet.id}: " + err_type)
            # end   
          else
            tweet.update_attributes(:tweeted_at => Time.zone.now)
          end
          alert_logger.info("-----------")
          sleep 1
        end
      end
    end
    alert_logger.info("===========")  
  end
  
  include PhotosHelper
  include ActionView::Helpers::TextHelper

  # Busca videos nuevos en las noticias del último mes, y los incluye en la WebTV.
  # Debería llamarse desde un cron job de la siguiente manera:
  # 36 * * * * sh /usr/app/ogov/batch_processes/include_new_videos_in_webtv.sh
  desc "Search new videos and include them in WebTV"
  task :include_new_videos_in_webtv do
    video_logger = Logger.new(File.dirname(__FILE__) + "/../../log/video_search.log") 
    video_logger.info("#{Time.zone.now.to_s(:short)}: Searching videos to include in WebTv")
    
    Document.find(:all, :select => "id, multimedia_path, title_es, title_eu, title_en, published_at",
      :conditions => "published_at IS NOT NULL AND coalesce(multimedia_dir, '') <> '' AND updated_at BETWEEN now()-'1month'::interval AND now()").each do |doc|
        list_of_videos = Dir.glob(doc.full_multimedia_path + "*.flv") + Dir.glob(doc.full_multimedia_path + "solo_irekia/*.flv")
        list_of_videos.each do |doc_video|
          candidate = doc_video.sub(/^#{Document::MULTIMEDIA_PATH}/, '').sub(/(_es|_eu|_en)*.flv$/, '')
          video_logger.info "\tChecking if video #{doc_video} exists... (#{candidate})"
          unless Video.exists?(:video_path => candidate)
            # This will work because Document::MULTIMEDIA_PATH and Video::VIDEO_PATH are the same directory
            # otherwise we would have to copy the contents from one place to another
            visibility = lang_visibility(candidate)
            video_logger.info "\tCreating video for #{candidate}. Visible in #{visibility.select {|k, v| v}.collect {|a| a[0]}.join(', ')}"
            new_video = Video.new(:video_path => candidate, :title_es => doc.title_es, 
              :title_eu => doc.title_eu.present? ? doc.title_eu : doc.title_es, :title_en => doc.title_en.present? ? doc.title_en : doc.title_es, 
              :show_in_es => visibility[:es], :show_in_eu => visibility[:eu], :show_in_en => visibility[:en],
              :published_at => doc.published_at, :document_id => doc.id)
            new_video.tag_list = doc.tag_list.dup
            if new_video.save
              video_logger.info "\tVideo has been saved. "
            else
              video_logger.info "\tThere were some errors saving video #{new_video.errors.full_messages.join(', ')}. "
            end
          else
            video_logger.info "\tVideo already exists. "
          end
          video_logger.info "----------"
        end
    end
    video_logger.info "=========="
  end

  # Comprueba si se han añadido idiomas a los videos de la WebTV.
  # Debería llamarse desde un cron job de la siguiente manera:
  # 43 * * * * sh /usr/app/ogov/batch_processes/check_webtv_video_languages.sh
  desc "Check language changes in WebTV videos of the last month"
  task :check_webtv_video_languages do
    video_logger = Logger.new(File.dirname(__FILE__) + "/../../log/video_lang_check.log") 
    video_logger.info("#{Time.zone.now.to_s(:short)}: Looking for new languages in WebTV videos...")
    Video.find(:all, :conditions => ["published_at BETWEEN ? AND ? ", Time.zone.now - 3.months, Time.zone.now]).each do |video|
      visibility = lang_visibility(video.video_path)

      #video.duration = video.duration_from_file if video.duration.blank?
      video.show_in_es = visibility[:es]
      video.show_in_eu = visibility[:eu]
      video.show_in_en = visibility[:en]
      video.save!
    end
    VideoSweeper.sweep
    Video::LANGUAGES.each do |lang|
      Net::HTTP.get(URI.parse("http://192.168.146.24:8800/#{lang}/podcast.xml"))
    end
  end
  
  
  # Recorre todos los videos de la WebTV buscando videos "huerfanos", es decir, entradas en la base de datos
  # que no tienen videos por detras bien porque los han borrado o los han renombrado.
  # Cuando los encuentra, lo pone en estado "draft" para que no se muestren en la web y los revisen a mano
  # Debería llamarse desde un cron job de la siguiente manera:
  # 45 * * * * sh /usr/app/ogov/batch_processes/hide_orphan_videos_in_webtv.sh
  desc "Looks for webtv entries without video files"
  task :hide_orphan_videos_in_webtv do
    video_logger = Logger.new(File.dirname(__FILE__) + "/../../log/orphan_videos.log") 
    video_logger.info("#{Time.zone.now.to_s(:short)}: Looking for videos to hide in WebTV...")
    
    Video.find(:all, :conditions => "draft<>'t'", :order => "id").each do |video|
      video_logger.info "\tChecking files for video #{video.id} (#{video.video_path}*.flv)"
      found_videos = Video.videos_in_dir(video.video_path)
      if found_videos.length > 0
        video_logger.info "\t#{found_videos.length} videos were found. "
      else
        video_logger.info "\tNo files found. Video marked as draft."
        video.update_attributes(:draft => true, :published_at => nil)
      end
      video_logger.info "----------"
    end
    video_logger.info "=========="
  end
  
  
  def lang_visibility(path)
    {:es => File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_es.flv") || (!File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_eu.flv") && !File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_en.flv")), 
     :eu => File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_eu.flv") || (!File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_es.flv") && !File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_en.flv")), 
     :en => File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_en.flv") || (!File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_es.flv") && !File.exists?("#{Document::MULTIMEDIA_PATH}#{path}_eu.flv"))}
  end
  
  
  desc "Importa las noticias de la aplicacion de Consejos de Gobierno"
  task :import_consejo_news do
    # require 'rss'
    consejo_logger = Logger.new(File.dirname(__FILE__) + "/../../log/consejo_imports.log")
    consejo_logger.info("#{Time.zone.now.to_s(:short)}: Importing consejo news...")
     sources = {"es" => "http://www.lehendakaritza.ejgv.euskadi.net/r48-acuerdos/es/v97aAcuerdosWar/V97AListaAcuerdosRSSServlet?idioma=c&tema=1&departamento=-1&accion=RSS&R01HNoPortal=true", 
                "eu" => "http://www.lehendakaritza.ejgv.euskadi.net/r48-acuerdos/es/v97aAcuerdosWar/V97AListaAcuerdosRSSServlet?idioma=e&tema=1&departamento=-1&accion=RSS&R01HNoPortal=true"}
    # sources = {"es" => "http://irekia.efaber.net/es.rss", 
    #           "eu" => "http://irekia.efaber.net/eu.rss"}
    sources.each do |lang, source|
      xml_data = Net::HTTP.get_response(URI.parse(source)).body
      begin
        feed = REXML::Document.new(xml_data)
      rescue => err
        consejo_logger.info("There were some errors importing consejo news: #{err.inspect}")
      else
        feed.elements.each('rss/channel/item') do |feed_news|
          news = News.find_or_initialize_by_consejo_news_id(feed_news.elements['guid'].text)
          consejo_logger.info("News: #{feed_news.elements['guid'].text}. It matches with #{news.id}")
          news.send("title_#{lang}=", "#{feed_news.elements['title'].text} (#{I18n.t('documents.consejo_de', :date => I18n.l(Date.parse(feed_news.elements['pubDate'].text), :locale => lang), :locale => lang)})")
          news_body = "<p class=\"r01Subtitular\">#{feed_news.elements['description'].text}</p>
          #{feed_news.elements['content'].text} 
          <!-- <p>#{I18n.t('shared.source')}: <a href=\"#{feed_news.elements['link'].text}\">euskadi.net</a></p> -->"
          feed_news.elements.each("enclosure") do |enc|
            news_body << "<p><a href=\"#{enc.attribute("url")}\">#{I18n.t('site.mas_informacion')}</a></p>"
          end
          news.send("body_#{lang}=", news_body)
          news.updated_by = User.irekia_robot.id
          
          if news.new_record?
            consejo_logger.info("Is new.")
            news.published_at = feed_news.elements['pubDate'].text
            news.tag_list.add ["Acuerdos del Consejo de Gobierno de #{I18n.localize(news.published_at.to_date, :locale => 'es')}", "Acuerdos del Consejo de Gobierno"]
            sanitized_category = feed_news.elements['category'].text.tildes.sub('Departamento de ', '').downcase.gsub(/[^a-z]/, '')
            
            case sanitized_category
            when 'administracionpublicayjusticiaxleg'
              dep_name = 'administracionpublica'
            when "presidenciaxleg"
              dep_name = 'lehendakaritzaxleg'
            else
              dep_name = sanitized_category
            end

            department = Department.find(:first, :conditions => ["regexp_replace(lower(tildes(name_es)), '[^a-z]', '', 'g') ~* ?", "^#{dep_name}.*"])

            news.organization_id = department.id
            news.created_by = User.irekia_robot.id
          end
          news.save!
        end
      end
      consejo_logger.info("------------")
      
      consejo_logger.info("============")
    end
  end
  
  
  # Busca fotos nuevas en las noticias del último mes, y los incluye en la Fototeca.
  # Debería llamarse desde un cron job de la siguiente manera:
  # 36 * * * * sh /usr/app/ogov/batch_processes/include_new_photos_in_gallery.sh
  desc "Search new photos and include them in Gallery"
  task :include_new_photos_in_gallery => :environment do
    include ActionController::UrlWriter
    default_url_options[:host] = RAILS_ENV.eql?("production") ? Notifier::SERVICEURL : "localhost:3000"
    
    photo_logger = Logger.new(File.dirname(__FILE__) + "/../../log/photo_search.log") 
    photo_logger.info("#{Time.zone.now.to_s(:short)}: Looking for new photos to include in gallery...")
    
    Document.find(:all, :select => "id, multimedia_path, title_es, title_eu, title_en, published_at",
      :conditions => "published_at IS NOT NULL AND coalesce(multimedia_dir, '') <> '' AND updated_at BETWEEN now()-'15days'::interval AND now()", 
      :order => "id DESC").each do |doc|
        photo_logger.info "Looking for photos for #{doc.id}..."
        list_of_photos = doc.photos
        if list_of_photos.length>0
          album = Album.find_or_initialize_by_document_id(doc.id)
          if album.new_record?
            album.title_es = doc.title_es
            album.title_eu = doc.title_eu
            album.title_en = doc.title_en
            album.tag_list = doc.tag_list.dup
            album.created_at = doc.published_at
          end
          list_of_photos.each do |doc_photo|
            candidate = doc_photo.sub(/^#{Document::MULTIMEDIA_PATH}/, '')
            photo_logger.info "\tChecking whether photo #{doc_photo} exists (#{candidate})..."
            unless Photo.exists?(:file_path => candidate)
              # This will work because Document::MULTIMEDIA_PATH and Video::PHOTO_PATH are the same directory
              # otherwise we would have to copy the contents from one place to another
              photo_logger.info "\tCreating photo for #{candidate}..."
              new_photo = album.photos.build(:file_path => candidate, :dir_path => Pathname.new(candidate).dirname,
                :title_es => doc.title_es, 
                :title_eu => doc.title_eu.present? ? doc.title_eu : doc.title_es, 
                :title_en => doc.title_en.present? ? doc.title_en : doc.title_es, 
                :created_at => doc.published_at,
                :document_id => doc.id)
              new_photo.tag_list = doc.tag_list.dup
            
              # Must ensure that all sizes are created
              Tools::Multimedia::PHOTOS_SIZES.each do |size, geometry|              
                begin
                  IrekiaThumbnail.make(doc_photo, geometry, size)
                rescue IrekiaThumbnailError => err
                  photo_logger.error err
                end
              end
            
              if new_photo.valid?
                photo_logger.info "\tPhoto is valid."
              else
                photo_logger.info "\tPhoto is not valid: #{new_photo.errors.full_messages.join(', ')}"
              end
            else
              photo_logger.info "\tPhoto already exists."
            end
            photo_logger.info "----------"
          end
          if album.save
            photo_logger.info "\tAlbum has been saved."
          else
            photo_logger.info "\tThere were some errors saving album #{album.errors.full_messages.join(', ')}. #{album.photos.collect {|p| p.errors.full_messages.join(', ')}.join("\n")}"
          end
        end
    end
    photo_logger.info "=========="
  end

  desc "Rebuilds thumbnails of photos of documents"
  task :build_photo_doc_thumbnails do
    photo_logger = Logger.new(File.dirname(__FILE__) + "/../../log/photo_rebuild_doc_thumbnails.log") 
    photo_logger.info("#{Time.zone.now.to_s(:short)}: Rebuilding all thumbnails for documents...")
    Document.find(:all, :order => "id desc").each do |doc|
      list_of_photos = Dir.glob("#{doc.full_multimedia_path}*.jpg") + Dir.glob("#{doc.full_multimedia_path}solo_irekia/*.jpg")
      list_of_photos.each do |doc_photo|
        build_thumbnails(doc_photo, Tools::Multimedia::PHOTOS_SIZES, photo_logger)
      end
    end
    photo_logger.info("#{Time.zone.now.to_s(:short)}: Finished")
    photo_logger.info("========")
  end
  
  desc "Rebuilds thumbnails of photos of gallery not belonging to news"
  task :build_photo_gallery_thumnails do
    photo_logger = Logger.new(File.dirname(__FILE__) + "/../../log/photo_rebuild_gallery_thumbnails.log") 
    photo_logger.info("#{Time.zone.now.to_s(:short)}: Rebuilding all thumbnails for gallery photos...")
    Photo.find(:all, :conditions => "document_id is null", :order => "id desc").each do |photo|
      build_thumbnails("#{Photo::PHOTOS_PATH}#{photo.file_path}", Tools::Multimedia::PHOTOS_SIZES, photo_logger)
    end
    photo_logger.info("#{Time.zone.now.to_s(:short)}: Finished")
    photo_logger.info("========")
  end
  
  def build_thumbnails(photo, sizes, logger)
    logger.info "Rebuilding thumbnails for #{photo}"
    dirname, filename = Pathname.new(photo).split
    if File.exists?(photo)
      sizes.each do |size, geometry|
        thumbnail_file = "#{dirname}/#{size}/#{filename}"
        if File.exists?(thumbnail_file)
          logger.info("skipping #{dirname}/#{size}/#{filename} because it already exists")
        else
          logger.info("creating #{dirname}/#{size}/#{filename}")
          begin
            IrekiaThumbnail.make(photo, geometry, size)
          rescue IrekiaThumbnailError => err
            logger.error err
          end
        end
      end
    end
    logger.info("---------------")
  end
  
  desc "Fix consejo de gobierno tags"
  task :fix_consejo_tags do
    News.find(:all, :conditions => "consejo_news_id IS NOT NULL").each do |item|
      tag = Tag.find_or_initialize_by_name_es("Acuerdos del Consejo de Gobierno de #{I18n.localize(item.published_at.to_date, :locale => 'es')}")
      if tag.new_record?
        tag.name_eu = "#{I18n.localize(item.published_at.to_date, :locale => 'eu')}ko Gobernu Kontseiluaren Akordioak"
        tag.save
      end
      item.tag_list.add [tag.name_es]
      
      puts "Adding tag #{tag.name_es} to #{item.id}"
      puts "Deleting tag #{item.tag_list.select {|t| t.match(/_acuerdos_/)}} from #{item.id}"
      item.tag_list.remove(item.tag_list.select {|t| t.match(/_acuerdos_/)})
      item.save
    end
  end

  desc "Delete tags from consejo news"
  task :delete_tags_from_consejo_news do
    list = News.tagged_with('portavoz, acuerdos, consejo de gobierno, consejo gobierno').find(:all, 
      # :select => "documents.id, (published_at+'3hour'::interval)::date, title_es, published_at",
      :conditions => "extract(dow from published_at+'3hour'::interval)=2 and consejo_news_id is null")
    list = list + News.find(:all, 
      # :select => "documents.id, (published_at+'3hour'::interval)::date, title_es, published_at",
      :conditions => "extract(dow from published_at+'3hour'::interval)=2 and title_es ilike '%consejo % gobierno%' and consejo_news_id is null")
      
    list.uniq!.sort! {|a,b| a.id <=> b.id}
    
    puts "Total: #{list.length} news"
    
    list.each do |item|
      tag = Tag.find_or_initialize_by_name_es("Acuerdos del Consejo de Gobierno de #{I18n.localize(item.published_at.to_date, :locale => 'es')}")
      tag.name_eu = "#{I18n.localize(item.published_at.to_date, :locale => 'eu')}ko Gobernu Kontseiluaren Akordioak"
      tag.name_en = "Agreement of the Council of Government on #{I18n.localize(item.published_at.to_date, :locale => 'en')}"
      if tag.new_record?
        puts "New tag: #{tag.name_es}"
        tag.save
      end

      item.body_es += "<p><a href=\"/es/tags/#{tag.sanitized_name_es}\">Acuerdos adoptados por el Consejo de Gobierno Vasco el #{I18n.localize(item.published_at.to_date, :locale => 'es')} </a></p>"
      item.body_eu += "<p><a href=\"/eu/tags/#{tag.sanitized_name_eu}\">Eusko Jaurlaritzaren Kontseiluak #{I18n.localize(item.published_at.to_date, :locale => 'eu')}n hartutako erabakiak </a></p>" unless item.body_eu.nil?
      item.body_en += "<p><a href=\"/en/tags/#{tag.sanitized_name_en}\">Agreements signed by the Council of Government of Basque country on #{I18n.localize(item.published_at.to_date, :locale => 'en')}</a></p>" unless item.body_en.nil?

      puts "Include link in #{item.id}, http://localhost:3000/es/news/#{item.id}"
      
      if item.save
        puts "ok!"
      else
        puts "Error including link in news #{item.id}: #{item.errors.inspect}"
      end
    end
  end
end
