#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class ProposalTest < ActiveSupport::TestCase

  test "title_es cannot be empty" do
    proposal = Proposal.new(default_values.merge(:title_es => ""))
    assert proposal.title_es.blank?
    assert !proposal.valid?
    assert proposal.errors.on(:title_es).include?("no puede estar vacío")      
  end

  test "user cannot be empty" do
    proposal = Proposal.new(default_values.merge(:user_id => nil))
    assert !proposal.valid?
    assert proposal.errors.on(:user).include?("no puede estar vacío")      
  end

  
  test "name is set to user name" do
    proposal = Proposal.new(default_values.merge(:name => ""))
    assert proposal.valid?
    assert_equal proposal.user.public_name, proposal.name
  end
  
  test "class_name is Proposal" do
    proposal = Proposal.new(default_values.merge(:name => ""))
    assert_equal "Proposal", proposal.class_name
  end

  test "should index to elasticsearch after save" do
    prepare_elasticsearch_test
    proposal = contributions(:approved_and_published_proposal)
    assert_deleted_from_elasticsearch proposal
    assert proposal.save
    assert_indexed_in_elasticsearch proposal
  end

  test "should delete from elasticsearch after destroy" do
    prepare_elasticsearch_test
    proposal = contributions(:approved_and_published_proposal)
    assert_deleted_from_elasticsearch proposal
    assert proposal.save
    assert_indexed_in_elasticsearch proposal
    assert proposal.destroy
    assert_deleted_from_elasticsearch proposal
  end
  
  context "visitante's new proposal" do
    setup do
      @proposal = Proposal.create(default_values.merge(:user_id => users(:visitante).id))
    end
    should "have pending status" do
      assert @proposal.pending?
    end
    
  end


  context "admin's new proposal" do
    setup do
      @proposal = Proposal.new(default_values.merge(:user_id => users(:admin).id))
    end
    should "not create proposal" do
      # NOTA: No se puede usar proposal.valid? porque la comprobación del tipo del usuario se hace before_create
      assert !@proposal.save
      assert_match /No puede crear/, @proposal.errors[:base]
    end
  end
  
  should "not approve proposal without organization" do
    @proposal = Proposal.create(default_values)
    assert @proposal.pending?
    @proposal.status = 'aprobado'
    assert !@proposal.valid?
    assert @proposal.errors.on(:base).include?("No puedes aprobar la propuesta sin asignarle departamento antes")
  end

  should "not let politicians create proposals" do
    @proposal = Proposal.create(default_values.merge(:user_id => users(:politician_one).id))
    # NOTA: No se puede usar proposal.valid? porque la comprobación del tipo del usuario se hace before_create
    assert !@proposal.save
    assert_match /No puede crear/, @proposal.errors[:base]
  end
  
  
  context "syncronization between proposal areas and it's comments areas" do
    setup do
      @proposal = contributions(:featured_proposal)
      @proposal.comments.create(:body => "thoughtful comment", :user => users(:comentador_oficial))
    end
    
    should "have lehendakaritza area tag" do
      assert_equal [areas(:a_lehendakaritza)], @proposal.areas
      @proposal.comments.each do |comment|
        assert_equal [areas(:a_lehendakaritza).area_tag], comment.tags
      end
    end
    
    context "via taggings" do
      should "add new area to comment" do
        @proposal.taggings.build(:tag => areas(:a_interior).area_tag)
        @proposal.save
        @proposal.reload
        assert @proposal.areas.include?(areas(:a_interior))
        @proposal.comments.each do |comment|
          assert comment.tags.include?(areas(:a_interior).area_tag)
        end
      end
      
      should "remove area from comment" do
        # in proposals area_id cannot be blank
        @proposal.taggings.create(:tag => areas(:a_interior).area_tag)
        @proposal.taggings.delete(@proposal.taggings.find_by_tag_id(areas(:a_lehendakaritza).area_tag.id))
        @proposal.save
        @proposal.reload
        assert !@proposal.areas.include?(areas(:a_lehendakaritza))
        @proposal.comments.each do |comment|
          assert !comment.tags.include?(areas(:a_lehendakaritza).area_tag)
        end
      end
      
      should "not sync tags that are not area tags" do
        new_tag = tags(:tag_politician_lehendakaritza)
        @proposal.taggings.build(:tag => new_tag)
        @proposal.save
        @proposal.reload
        assert @proposal.tags.include?(new_tag)
        @proposal.comments.each do |comment|
          assert !comment.tags.include?(new_tag)
        end
      end
    end

    context "via area_tags=" do
      # This is what the form in admin/documents/edit_tags uses
      should "add new area to comment" do
        @proposal.area_tags= "#{areas(:a_lehendakaritza).area_tag.name_es}, #{areas(:a_interior).area_tag.name_es}"
        assert @proposal.save
        @proposal.reload
        assert @proposal.areas.include?(areas(:a_interior))
        @proposal.comments.each do |comment|
          assert comment.tags.include?(areas(:a_interior).area_tag)
        end
      end
      
      should "remove area from comment" do
        @proposal.area_tags = "#{areas(:a_interior).area_tag.name_es}"
        @proposal.save
        @proposal.reload
        assert !@proposal.areas.include?(areas(:a_lehendakaritza))
        @proposal.comments.each do |comment|
          assert !comment.tags.include?(areas(:a_lehendakaritza).area_tag)
        end
      end
      
      should "not sync tags that are not area tags" do
        new_tag = tags(:tag_politician_lehendakaritza)
        @proposal.area_tags = "#{new_tag.name_es}"
        @proposal.save
        @proposal.reload
        assert @proposal.tags.include?(new_tag)
        @proposal.comments.each do |comment|
          assert !comment.tags.include?(new_tag)
        end
      end
      
    end

    context "via tag_list" do
      should "add new area to comment" do
        @proposal.tag_list.add areas(:a_interior).area_tag.name_es
        @proposal.save
        @proposal.reload
        assert @proposal.areas.include?(areas(:a_interior))
        @proposal.comments.each do |comment|
          assert comment.tags.include?(areas(:a_interior).area_tag)
        end
      end
    
      should "remove area from comment" do
        puts areas(:a_lehendakaritza).area_tag.name_es
        @proposal.tag_list.remove areas(:a_lehendakaritza).area_tag.name_es
        @proposal.save
        @proposal.reload
        assert !@proposal.areas.include?(areas(:a_lehendakaritza))
        @proposal.comments.each do |comment|
          assert !comment.tags.include?(areas(:a_lehendakaritza).area_tag)
        end
      end
      
      should "not sync tags that are not area tags" do
        new_tag = tags(:tag_politician_lehendakaritza)
        @proposal.taggings.build(:tag => new_tag)
        @proposal.save
        @proposal.reload
        assert @proposal.tags.include?(new_tag)
        @proposal.comments.each do |comment|
          assert !comment.tags.include?(new_tag)
        end
      end
    end
  end
  

  def default_values
    {:email => "test@example.com", :body_es => "Body", :title_es => "Titulo", :area_id => areas(:a_lehendakaritza).id, :user_id => users(:visitante).id}
  end
end
