class Tag < ActiveRecord::Base
  has_many :taggings, :dependent => :destroy       
  belongs_to :criterio
  translatable_columns :name, :sanitized_name
  Tag::PUBLIC_CONDITIONS = "name_es NOT LIKE E'\\\\_%'"
  Tag::PRIVATE_CONDITIONS = "name_es LIKE E'\\\\_%'"
  Tag::POLITICIANS_CONDITIONS = "kind =  E'Político'"
  named_scope :public, :conditions => Tag::PUBLIC_CONDITIONS
  named_scope :private, :conditions => Tag::PRIVATE_CONDITIONS
  named_scope :politicians, :conditions => Tag::POLITICIANS_CONDITIONS

  validates_presence_of :name_es, :name_eu, :name_en
  
  # Si traducimos un tag, hay que traducir el indice de todos los documentos taggeados con eso
  after_update :reindex_tagged_documents      
  
  def reindex_tagged_documents
    if name_es_changed? || name_eu_changed? || name_en_changed?
      Document.find_tagged_with(self.name_es).each do |doc|
        # doc.save
        # eli@efaber.net, 4-06-2012.
        # temporalmente, mientras están poniendo al día los datos, sólo indexamos el documento
        # sin pasar por el save porque tarda mucho y cuelga el mongrel que le ha tocado.
        doc.update_elasticsearch_server if doc.respond_to?(:update_elasticsearch_server)
      end
    end
  end          
               
  # GC related stuff
  def gc_link
    l = nil
    if self.kind_info.present?
      l = case self.kind
        when 'Persona'
          "http://www2.irekia.euskadi.net/#{I18n.locale}/people/#{self.kind_info.to_i}"
        when 'Entidad'
          "http://www2.irekia.euskadi.net/#{I18n.locale}/entities/#{self.kind_info.to_i}"
      end
    end
    return l
  end

  def translated_to?(locale)
    res = self.translated? || locale.eql?("es")
    if !res
      txt = self.send("name_#{locale}")
      res = (txt.eql?(self.name_es) || txt.blank?) ? false : true 
    end

    res
  end           
  
  def criterio_title
    "tags: #{self.name_en}|#{self.name_es}|#{self.name_eu}"
  end
    
  Tag::LANGUAGES = [:es, :eu, :en]
  
  def Tag.name_columns
    Tag::LANGUAGES.collect {|l| "name_#{l}"}
  end
  
  def Tag.sanitized_name_columns
    Tag::LANGUAGES.collect {|l| "sanitized_name_#{l}"}
  end

  # Not necessary now that we can merge tags
  # Tag.name_columns.each do |c|
  #   validates_uniqueness_of c
  # end
  
  validate :validates_presence_of_any_language
  def validates_presence_of_any_language
    names_empty = Tag.name_columns.collect {|c| c if self.send(c).blank?}.compact
    if names_empty.length == Tag.name_columns.length
      Tag.name_columns.each do |t|
        errors.add t, "El nombre no puede estar vacío"
      end
    else
      true
    end
  end
  
  before_validation :set_name_in_every_language
  before_save :set_sanitized_names
  # Temporaly disabled because doesn't work in v4
  # before_save :create_associated_criterio
  before_update :set_as_translated_if_every_language_is_different
  
  def set_name_in_every_language
    nonempty_names = Tag.name_columns.collect {|c| self.send(c) unless self.send(c).blank?}.compact
    Tag::LANGUAGES.each do |l|
      self.send("name_#{l}=", nonempty_names[0]) if self.send("name_#{l}").blank?
    end
  end
  
  def set_sanitized_names
    Tag::LANGUAGES.each do |l|
      self.send("sanitized_name_#{l}=", self.send("name_#{l}").to_tag)
    end
  end
  
  # Para cada tag crear el criterio asociado, solo si es visible
  def create_associated_criterio
    if self.name_es.match(/^_/).nil?
      if self.criterio.present? && (self.name_es_changed? || self.name_eu_changed? || self.name_en_changed?)
        self.criterio.update_attribute(:title, self.criterio_title)
      else             
        self.create_criterio(:title => self.criterio_title, :ip => '127.0.0.1')
      end
    end  
  end  
  
  def all_languages_different
    Tag.name_columns.collect {|c| self.send(c)}.uniq.length == Tag.name_columns.length
  end
  
  def set_as_translated_if_every_language_is_different
    if all_languages_different
      self.translated = true 
    else
      true
    end
  end
  
  # we want to have the tag name in urls
  def to_param
    sanitized_name
  end
  
  cattr_accessor :destroy_unused
  self.destroy_unused = true
  
  # LIKE is used for cross-database case-insensitivity
  def self.find_or_create_with_like_by_name(name_es)
    find(:first, :conditions => ["name_es LIKE ?", name_es]) || create(:name_es => name_es)
  end
  
  def ==(object)
    super || (object.is_a?(Tag) && name == object.name)
  end
  
  def to_s
    name
  end
  
  def count
    read_attribute(:count).to_i
  end
  
  class << self
    # Calculate the tag counts for all tags.
    #  :start_at - Restrict the tags to those created after a certain time
    #  :end_at - Restrict the tags to those created before a certain time
    #  :conditions - A piece of SQL conditions to add to the query
    #  :limit - The maximum number of tags to return
    #  :order - A piece of SQL to order by. Eg 'count desc' or 'taggings.created_at desc'
    #  :at_least - Exclude tags with a frequency less than the given value
    #  :at_most - Exclude tags with a frequency greater than the given value
    def counts(options = {})
      find(:all, options_for_counts(options))
    end
    
    def options_for_counts(options = {})
      options.assert_valid_keys :start_at, :end_at, :conditions, :at_least, :at_most, :order, :limit, :joins
      options = options.dup
      
      start_at = sanitize_sql(["#{Tagging.table_name}.created_at >= ?", options.delete(:start_at)]) if options[:start_at]
      end_at = sanitize_sql(["#{Tagging.table_name}.created_at <= ?", options.delete(:end_at)]) if options[:end_at]
      
      conditions = [
        options.delete(:conditions),
        start_at,
        end_at
      ].compact
      
      conditions = conditions.any? ? conditions.join(' AND ') : nil
      
      joins = ["INNER JOIN #{Tagging.table_name} ON #{Tag.table_name}.id = #{Tagging.table_name}.tag_id"]
      joins << options.delete(:joins) if options[:joins]
      
      at_least  = sanitize_sql(['COUNT(*) >= ?', options.delete(:at_least)]) if options[:at_least]
      at_most   = sanitize_sql(['COUNT(*) <= ?', options.delete(:at_most)]) if options[:at_most]
      having    = [at_least, at_most].compact.join(' AND ')
      group_by  = "#{Tag.table_name}.id, #{Tag.table_name}.#{Tag.column_translated('name')}, 
                   #{Tag.table_name}.#{Tag.column_translated('sanitized_name')} HAVING COUNT(*) > 0"
      group_by << " AND #{having}" unless having.blank?
      
      { :select     => "#{Tag.table_name}.id, #{Tag.table_name}.#{Tag.column_translated('name')}, 
                        #{Tag.table_name}.#{Tag.column_translated('sanitized_name')}, COUNT(*) AS count", 
        :joins      => joins.join(" "),
        :conditions => conditions,
        :group      => group_by,
        :order      => Tag.column_translated('name')
      }.update(options)
    end
    
    def find_for_page(content_method)
      find_all_by_sanitized_name_es(content_method)
    end

    def duplicated_tags
      #find_by_sql("SELECT distinct on (t1.id) t1.id, t1.name_es FROM tags t1, tags t2 
      #  WHERE t1.id<>t2.id AND t1.name_es = t2.name_es AND t1.name_eu = t2.name_eu AND t1.name_en = t2.name_en")
      find_by_sql("SELECT distinct on (t1.id) t1.id, t1.name_es, t1.sanitized_name_es, t1.name_eu, t1.sanitized_name_eu, t1.kind, t1.kind_info FROM tags t1, tags t2 
         WHERE t1.id<>t2.id AND t1.sanitized_name_es = t2.sanitized_name_es AND t1.sanitized_name_eu = t2.sanitized_name_eu AND t1.sanitized_name_en = t2.sanitized_name_en")
    end
    
  end
end
